# -*- coding: utf-8 -*-
"""
The Blackboard Integrated Channel package.
"""

__version__ = "0.0.1"

default_app_config = ("integrated_channels.blackboard.apps."
                      "BlackboardConfig")
