# Generated by Django 2.2.15 on 2020-08-06 17:56

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise', '0107_remove_branding_config_banner_fields'),
    ]

    operations = [
        migrations.AddField(
            model_name='historicallicensedenterprisecourseenrollment',
            name='is_revoked',
            field=models.BooleanField(default=False, help_text="Whether the licensed enterprise course enrollment is revoked, e.g., when a user's license is revoked."),
        ),
        migrations.AddField(
            model_name='licensedenterprisecourseenrollment',
            name='is_revoked',
            field=models.BooleanField(default=False, help_text="Whether the licensed enterprise course enrollment is revoked, e.g., when a user's license is revoked."),
        ),
    ]
