# Generated by Django 2.2.19 on 2021-08-03 18:54

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise', '0138_bulkcatalogqueryupdatecommandconfiguration'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='adminnotification',
            options={'ordering': ('start_date',), 'verbose_name': 'Admin Notification', 'verbose_name_plural': 'Admin Notifications'},
        ),
        migrations.AddField(
            model_name='adminnotification',
            name='title',
            field=models.CharField(default='New Notification', help_text='Notification banner title which will appear to enterprise admin on admin portal.', max_length=255),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='adminnotification',
            name='text',
            field=models.CharField(help_text='Notification banner text which will appear to enterprise admin on admin portal.', max_length=255),
        ),
    ]
